// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "VNSceneGen.hpp"

using namespace Dawn;

void VNSceneGen::generate(
  std::vector<std::string> *out,
  struct VNScene *scene,
  std::string tabs
) {
  struct ClassGenInfo classInfo;
  classInfo.clazz = "TestScene";
  classInfo.extend = "Scene";
  classInfo.constructorArgs = "DawnGame *game";
  classInfo.extendArgs = "game";

  classInfo.includes.push_back("scene/Scene.hpp");

  struct MethodGenInfo methodAssets;
  methodAssets.name = "getRequiredAssets";
  methodAssets.isOverride = true;
  methodAssets.type = "std::vector<Asset*>";
  line(&methodAssets.body, "auto assMan = &this->game->assetManager;", "");
  line(&methodAssets.body, "std::vector<Asset*> assets;", "");
  line(&methodAssets.body, "return assets;", "");

  struct MethodGenInfo methodStage;
  methodStage.name = "stage";
  methodStage.isOverride = "true";
  line(&methodStage.body, "// test", "");

  // TEST
  line(&methodStage.body, "auto canvas = UICanvas::create(this);", "");
  line(&methodStage.body, "", "");

  classInfo.includes.push_back("scene/components/display/Camera.hpp");
  line(&methodStage.body, "auto camera = Camera::create(this);", "");
  line(&methodStage.body, "camera->fov = 0.436332f;", "");
  line(&methodStage.body, "camera->transform->lookAt(glm::vec3(10, 10, 10), glm::vec3(0, 0, 0));", "");

  // Items
  int32_t itemRefIndex = 0;
  auto itItems = scene->items.begin();
  while(itItems != scene->items.end()) {
    struct VNSceneItem item = *itItems;
    if(item.ref.empty()) item.ref = "item" + std::to_string(itemRefIndex++);
    classInfo.includes.push_back(item.prefab + ".hpp");
    
    line(&methodStage.body, "", "");
    line(&methodStage.body, "auto " + item.ref + " = " + item.className + "::create(this);", "");

    ++itItems;
  }

  line(&methodStage.body, "", "");

  // Events
  classInfo.includes.push_back("games/vn/components/VNManager.hpp");
  line(&methodStage.body, "auto vnItem = this->createSceneItem();", "");
  line(&methodStage.body, "auto vnManager = vnItem->addComponent<VNManager>();", "");
  line(&methodStage.body, "VNEvent *previous = vnManager->createEvent<VNDummyEvent>();", "");
  line(&methodStage.body, "auto eventStart = previous;", "");

  int32_t eventIndex = 0;
  auto itEvents = scene->events.events.begin();
  while(itEvents != scene->events.events.end()) {
    std::string eventName = "event" + std::to_string(eventIndex);
    std::string initType = "";
    std::string initArgs = "";
    std::string toInclude = "";
    std::string prev = "previous";
    std::vector<std::string> afterLines;

    switch(itEvents->type) {
      case VN_SCENE_EVENT_TYPE_TEXT:
        initType = "VNTextEvent";
        toInclude = "games/vn/events/VNTextEvent.hpp";
        line(&afterLines, eventName + "->" + "text = \"" + itEvents->text.texts.begin()->text + "\";", "");
        break;

      case VN_SCENE_EVENT_TYPE_POSITION:
        initType = "VNPositionEvent";
        toInclude = "games/vn/events/VNPositionEvent.hpp";
        line(&afterLines, eventName + "->item = " + itEvents->position.item + ";", "");
        if(itEvents->position.x != "") line(&afterLines, eventName + "->" + "to.x = " + itEvents->position.x + ";", "");
        if(itEvents->position.y != "") line(&afterLines, eventName + "->" + "to.y = " + itEvents->position.y + ";", "");
        if(itEvents->position.z != "") line(&afterLines, eventName + "->" + "to.z = " + itEvents->position.z + ";", "");
        break;

      case VN_SCENE_EVENT_TYPE_SET:
        initType = "VNSetEvent<" + itEvents->set.type + ">";
        toInclude = "games/vn/events/VNSetEvent.hpp";
        line(&afterLines, eventName + "->modifies = &" + itEvents->set.property + ";", "");
        line(&afterLines, eventName + "->to = " + itEvents->set.to + ";", "");
        if(itEvents->set.from != "") line(&afterLines, eventName + "->from = " + itEvents->set.from + ";", "");
        if(itEvents->set.duration != "") line(&afterLines, eventName + "->duration = " + itEvents->set.duration + ";", "");
        break;


      default:
        std::cout << "Unknown event type: " << itEvents->type << std::endl;
        assertUnreachable();
    }

    if(!toInclude.empty()) classInfo.includes.push_back(toInclude);
    line(&methodStage.body, "", "");
    line(
      &methodStage.body,
      "auto " + eventName + " = " + prev + "->then(vnManager->createEvent<" + initType + ">(" + initArgs + "));",
      ""
    );
    line(&methodStage.body, "previous = " + eventName + ";", "");
    lines(&methodStage.body, afterLines, "");
    
    eventIndex++;
    ++itEvents;
  }

  line(&methodStage.body, "", "");
  line(&methodStage.body, "vnManager->setEvent(eventStart);", "");

  // Add in methods
  CodeGen::methodGen(&classInfo.protectedCode, methodAssets);
  line(&classInfo.protectedCode, "", "");
  CodeGen::methodGen(&classInfo.protectedCode, methodStage);

  CodeGen::classGen(out, classInfo);
}